<?php

require_once dirname(__FILE__).'/../lib/articleGeneratorConfiguration.class.php';
require_once dirname(__FILE__).'/../lib/articleGeneratorHelper.class.php';

/**
 * article actions.
 *
 * @package    sf_sandbox
 * @subpackage article
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class articleActions extends autoArticleActions
{
	public function executeEdit(sfWebRequest $request)
	{
		$this->article = $this->getRoute()->getObject();

		$this->article->setPrixVenteHT(Utils::doubleEnToFr($this->article->getPrixVenteHT()));
		$this->article->setTauxRemiseFournisseur(Utils::doubleEnToFr($this->article->getTauxRemiseFournisseur()));
		$this->article->setPrixVenteTHS(Utils::doubleEnToFr($this->article->getPrixVenteTHS()));
		$this->article->setMargePourcentage(Utils::doubleEnToFr($this->article->getMargePourcentage()));
		$this->article->setMargeEuro(Utils::doubleEnToFr($this->article->getMargeEuro()));
		$this->article->setPrixTTC(Utils::doubleEnToFr($this->article->getPrixTTC()));

		$this->form = $this->configuration->getForm($this->article);
	}

	protected function processForm(sfWebRequest $request, sfForm $form)
	{
		$values = $request->getParameter($form->getName());
		$values["prix_vente_ht"] = Utils::doubleFrToEn($values["prix_vente_ht"]);
		$values["taux_remise_fournisseur"] = Utils::doubleFrToEn($values["taux_remise_fournisseur"]);
		$values["prix_vente_ths"] = Utils::doubleFrToEn($values["prix_vente_ths"]);
		$values["marge_pourcentage"] = Utils::doubleFrToEn($values["marge_pourcentage"]);
		$values["marge_euro"] = Utils::doubleFrToEn($values["marge_euro"]);
		$values["prix_ttc"] = Utils::doubleFrToEn($values["prix_ttc"]);
		
		$form->bind($values, $request->getFiles($form->getName()));
		if ($form->isValid())
		{
			// Balise meta
			$famille = Doctrine::getTable("Famille")->find($values["id_famille"]);
			$values["balise_meta"] = $famille->getLibelle().' - '.$values["designation_detaillee"];
		
			$form->bind($values, $request->getFiles($form->getName()));
			
			$notice = $form->getObject()->isNew() ? 'The item was created successfully.' : 'The item was updated successfully.';

			try
			{
				$article = $form->save();
			} 
			catch (Doctrine_Validator_Exception $e) 
			{

				$errorStack = $form->getObject()->getErrorStack();

				$message = get_class($form->getObject()) . ' has ' . count($errorStack) . " field" . (count($errorStack) > 1 ?  's' : null) . " with validation errors: ";
				foreach ($errorStack as $field => $errors)
				{
					$message .= "$field (" . implode(", ", $errors) . "), ";
				}
				$message = trim($message, ', ');

				$this->getUser()->setFlash('error', $message);
				return sfView::SUCCESS;
			}

			$this->dispatcher->notify(new sfEvent($this, 'admin.save_object', array('object' => $article)));

			if ($request->hasParameter('_save_and_add'))
			{
				$this->getUser()->setFlash('notice', $notice.' You can add another one below.');

				$this->redirect('@article_new');
			}
			else
			{
				$this->getUser()->setFlash('notice', $notice);

				$this->redirect(array('sf_route' => 'article_edit', 'sf_subject' => $article));
			}
		}
		else
		{
			$this->getUser()->setFlash('error', 'The item has not been saved due to some errors.', false);
		}
	}

	public function executeIndex(sfWebRequest $request)
	{
		$this->max_per_page = 1000;
		if($this->getUser()->hasAttribute('max_per_page_article'))
		{
			if($this->configuration->getPagerMaxPerPage() != $this->getUser()->getAttribute('max_per_page_article'))
			{
				$this->setPage(1);
			}
			
			$this->max_per_page = $this->getUser()->getAttribute('max_per_page_article');
		}
		
		$this->configuration->setPagerMaxPerPage($this->max_per_page);

		// sorting
		if ($request->getParameter('sort') && $this->isValidSortColumn($request->getParameter('sort')))
		{
			$this->setSort(array($request->getParameter('sort'), $request->getParameter('sort_type')));
		}
		
		// pager
		if ($request->getParameter('page'))
		{
			$this->setPage($request->getParameter('page'));
		}

		$this->pager = $this->getPager();
		$this->sort = $this->getSort();
			
		// Formulaire article import
		$this->form = new ArticleImportForm();
		
		// Permet de récupérer la valeur de reference_ths_sage pour le filtre
		$filters = $this->getFilters();
		if(isset($filters['reference_ths_sage']))
		{
			$this->filters->setDefault('reference_ths_sage', $filters['reference_ths_sage']);
		}
	}
	
	public function executeImport(sfWebRequest $request)
	{
		if($request->isMethod("POST"))
		{
			$form = new ArticleImportForm();
			$form->bind($request->getParameter('article_import'), $request->getFiles('article_import'));
			if($form->isValid())
			{
				$file = $form->getValue('file');

				$filename = 'uploaded_'.sha1($file->getOriginalName());
				$extension = $file->getExtension($file->getOriginalExtension());
				$file->save(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);

				// Récupère la liste des familles d'articles
				$listeFamilleArticle = FamilleTable::recupererListeArticleFamilleEnTableau();

				//Récupère les données
				$fichier = sfConfig::get('sf_upload_dir').'/'.$filename.$extension;

				try
				{
					// Vérification que les images existent bien
					$repertoireImage = sfConfig::get('import_dir').DIRECTORY_SEPARATOR;
					$repertoireArticle = sfConfig::get('article_image_dir').DIRECTORY_SEPARATOR;
					$repertoireArticleMiniature = sfConfig::get('article_miniature_dir').DIRECTORY_SEPARATOR;

					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							if(!file_exists($repertoireImage.$data[26].".jpg"))
							{
								throw new Exception("Aucune image n'est associé à la référence THS '$data[26]' ".$repertoireImage.$data[26].".jpg");
							}
						}
						fclose($handle);
					}

					// Vérification que tous les articles ont une famille
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							if($data[27] == "")
							{
								throw new Exception("La référence THS '$data[26]' ne contient pas de famille");
							}

							if(!isset($listeFamilleArticle[strtolower($data[27])]))
							{
								throw new Exception("La référence THS '$data[26]' contient une famille qui n'existe pas");
							}
						}
						fclose($handle);
					}

					// Vérification des références
					$listeReference = array();
					$listeReferenceTHS = array();
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							$listeReferenceTHS[] = $data[26];
							$listeReference[$data[26]] = $data[3];
						}
						fclose($handle);
					}

					try
					{
						$connection = Doctrine_Manager::connection();
						$connection->beginTransaction();

						if(sizeof($listeReferenceTHS) > 0)
						{
							// On récupère toutes les références existantes et on compare le couple reference_ths / reference_ths_sage
							// Si un couple ne correspond, on termine l'import
							$listeArticle = ArticleTable::recupererListeArticlePourImport($listeReferenceTHS);
							foreach($listeArticle as $article)
							{
								if($listeReference[$article->reference_ths] != $article->reference_ths_sage)
								{
									throw new Exception("La référence THS '$article->reference_ths' ne contient pas la même référence sage");
								}
							}

							// On efface toutes les références que l'on va importer si elle existe déjà en base de données
							// ArticleTable::supprimerArticle($listeReferenceTHS);
						}

						if (($handle = fopen($fichier, "r")) !== FALSE)
						{
							while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
							{
								$objet = Doctrine_Core::getTable("Article")->find($data[26]);
								
								if(!($objet instanceof Article))
								{
									$objet = new Article();
								}
								
								$objet->reference_ths = $data[26];
								$objet->reference_ths_sage = $data[3];
								$objet->id_famille = $listeFamilleArticle[strtolower($data[27])];
								$objet->designation_courte = $data[29];
								$objet->designation_detaillee = $data[28];
								$objet->specificite = $data[31];
								$objet->dimensions_a = $data[36];
								$objet->dimensions_b = $data[37];
								$objet->matiere = $data[39];
								$objet->pression_maximum = $data[40];
								$objet->temperature_maximum = $data[41];
								$objet->diametre_nominal = $data[38];
								$objet->reference_metrique_entree = $data[42];
								$objet->sexe_entree = $data[43];
								$objet->precision_supplementaire_entree = $data[44];
								$objet->reference_metrique_sortie = $data[45];
								$objet->sexe_sortie = $data[46];
								$objet->precision_supplementaire_sortie = $data[47];
								$objet->calibre_piece = $data[49];
								$objet->debit_max = $data[50];
								$objet->hors_gabarit = $data[51];
								$objet->piece_origine = $data[54];
								$objet->piece_visible = $data[55];
								$objet->poids = str_replace(",", ".", $data[35]);
								$objet->prix_vente_ht = str_replace(",", ".", $data[11]);
								$objet->taux_remise_fournisseur = str_replace(",", ".", $data[6]);
								$objet->prix_vente_ths = str_replace(",", ".", $data[11]);
								$objet->marge_pourcentage = str_replace(",", ".", $data[9]);
								$objet->marge_euro = str_replace(",", ".", $data[12]);
								$objet->prix_ttc = str_replace(",", ".", $data[11]) * 1.196;
								$objet->stock = $data[13];
								$objet->url_article = "http://www.haute-pression.com/".Utils::slugify($data[27])."/".Utils::slugify($data[26]).".html";
								$objet->balise_title = $data[29];
								$objet->balise_meta = strtolower($data[27]).' - '.$data[28];
								$objet->minimum_commande = $data[52];
								$objet->save();
							}
							fclose($handle);
						}
						$connection->commit();
					}
					catch(Exception $e)
					{
						$connection->rollback();
						throw new Exception($e->getMessage());
					}

					// Création des miniatures
					if (($handle = fopen($fichier, "r")) !== FALSE)
					{
						while (($data = fgetcsv($handle, 0, ";")) !== FALSE)
						{
							// Création de la miniature
							$thumbnail = new sfThumbnail(100);
							$thumbnail->loadFile($repertoireImage.$data[26].".jpg");
							$thumbnail->save($repertoireArticleMiniature.$data[26]."_100.jpg");

							// on déplace les originaux
							copy($repertoireImage.$data[26].".jpg", $repertoireArticle.$data[26].".jpg");

							// unlink($repertoireImage.$data[26].".jpg");
						}
						fclose($handle);
					}

					$this->getUser()->setFlash('notice', 'L\'importation s\'est déroulé avec succès');
				}
				catch(Exception $e)
				{
					unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
					$this->getUser()->setFlash('error', $e->getMessage());
					$this->redirect("@article");
				}
			}
			else
			{
				$this->getUser()->setFlash('error', 'Erreur lors de l\'importation');
			}
		}
		unlink(sfConfig::get('sf_upload_dir').'/'.$filename.$extension);
		$this->redirect("@article");
	}

	public function executeBatch(sfWebRequest $request)
	{
		$request->checkCSRFProtection();

		if($request->hasParameter("max_per_page_article") && !$action = $request->getParameter('batch_action'))
		{
			$this->getUser()->setAttribute('max_per_page_article', $request->getParameter("max_per_page_article"));
			$this->redirect('@article');
		}

		if (!$ids = $request->getParameter('ids'))
		{
			$this->getUser()->setFlash('error', 'You must at least select one item.');

			$this->redirect('@article');
		}

		if (!$action = $request->getParameter('batch_action'))
		{
			$this->getUser()->setFlash('error', 'You must select an action to execute on the selected items.');

			$this->redirect('@article');
		}

		if (!method_exists($this, $method = 'execute'.ucfirst($action)))
		{
			throw new InvalidArgumentException(sprintf('You must create a "%s" method for action "%s"', $method, $action));
		}

		if (!$this->getUser()->hasCredential($this->configuration->getCredentials($action)))
		{
			$this->forward(sfConfig::get('sf_secure_module'), sfConfig::get('sf_secure_action'));
		}

		$validator = new sfValidatorDoctrineChoice(array('multiple' => true, 'model' => 'Article'));
		try
		{
			// validate ids
			$ids = $validator->clean($ids);

			// execute batch
			$this->$method($request);
		}
		catch (sfValidatorError $e)
		{
			$this->getUser()->setFlash('error', 'A problem occurs when deleting the selected items as some items do not exist anymore.');
		}

		$this->redirect('@article');
	}
	
	public function executeRecupererArticlePourAssociation(sfWebRequest $request)
	{
		$reference_ths_sage = $request->getPostParameter('reference_ths_sage');
		if ($request->isXmlHttpRequest())
		{
			$donnees = array();
			$donnees['reference_ths_sage'] = "";
			$donnees['reference_ths'] = "";
			$donnees['designation'] = "";
			$donnees['prix'] = "";
			$donnees['poids'] = "";
			
			$article = Doctrine::getTable("Article")->findOneByReferenceThsSage($reference_ths_sage);
			
			if($article instanceof Article)
			{
				$donnees = array();
				$donnees['reference_ths_sage'] = $article->getReferenceThsSage();
				$donnees['reference_ths'] = $article->getReferenceThs();
				$donnees['designation'] = $article->getDesignationCourte();
				$donnees['prix'] = $article->getPrixVenteThs();
				$donnees['poids'] = $article->getPoids();
			}
			else
			{
				$article = Doctrine::getTable("Article")->findOneByReferenceThs($reference_ths_sage);
				
				if($article instanceof Article)
				{
					$donnees = array();
					$donnees['reference_ths_sage'] = $article->getReferenceThsSage();
					$donnees['reference_ths'] = $article->getReferenceThs();
					$donnees['designation'] = $article->getDesignationCourte();
					$donnees['prix'] = $article->getPrixVenteThs();
					$donnees['poids'] = $article->getPoids();
				}
			}
			
			if(!($article instanceof Article))
			{
				$nettoyeur = Doctrine::getTable("NettoyeurHautePression")->findOneByReferenceThs($reference_ths_sage);
				
				if($nettoyeur instanceof NettoyeurHautePression)
				{
					$donnees = array();
					$donnees['reference_ths_sage'] = $nettoyeur->getReferenceConstructeur();
					$donnees['reference_ths'] = $nettoyeur->getReferenceThs();
					$donnees['designation'] = $nettoyeur->getMarque()->getLibelle()." ".$nettoyeur->getModele();
					$donnees['prix'] = $nettoyeur->getPrixVenteThs();
					$donnees['poids'] = $nettoyeur->getPoids();
				}
				else
				{
					$nettoyeur = Doctrine::getTable("NettoyeurHautePression")->findOneByReferenceConstructeur($reference_ths_sage);
				
					if($nettoyeur instanceof NettoyeurHautePression)
					{
						$donnees = array();
						$donnees['reference_ths_sage'] = $nettoyeur->getReferenceConstructeur();
						$donnees['reference_ths'] = $nettoyeur->getReferenceThs();
						$donnees['designation'] = $nettoyeur->getMarque()->getLibelle()." ".$nettoyeur->getModele();
						$donnees['prix'] = $nettoyeur->getPrixVenteThs();
						$donnees['poids'] = $nettoyeur->getPoids();
					}
				}
			}
			
			return $this->renderText(json_encode($donnees));
		}
		return sfView::NONE;
	}
}
