<?php

class Utils
{
	static public function slugify($text)
	{
		// replace non letter or digits by -
		$text = preg_replace('~[^\\pL\d]+~u', '-', $text);

		// trim
		$text = trim($text, '-');

		// transliterate
		if (function_exists('iconv'))
		{
			$text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);
		}

		// lowercase
		$text = strtolower($text);

		// remove unwanted characters
		$text = preg_replace('~[^-\w]+~', '', $text);

		if (empty($text))
		{
			return 'n-a';
		}

		return $text;
	}
	
	/**
	 * Formate une chaîne de caractère pour une URL
	 * @param String $text
	 * @return String
	 */
	static public function stringForUrl($text)
	{
		return self::slugify(strtolower(trim($text)));
	}
	
	/**
	 * Formate un prix pour avoir un format par exemple 300,00.
	 * @param unknown_type $prix
	 */
	static public function formatDePrix($prix)
	{
		if(!strpos($prix, "."))
		{
			$prix .= ".00";
		}
		return self::doubleEnToFr($prix);
	}
	
	/**
	 * 
	 * Enter description here ...
	 * @param unknown_type $prix
	 * @param unknown_type $tauxTVA - format : 19.6
	 */
	static public function prixAvecTVA($prix, $tauxTVA)
	{
		return round($prix * (1 + $tauxTVA / 100), 2);
	}
	
	static public function doubleEnToFr($double)
	{
		return str_replace(".", ",", $double);
	}
	
	static public function doubleFrToEn($double)
	{
		return str_replace(",", ".", $double);
	}
	
	public static function dateEnToFr($date)
	{
		$dateSortie = substr($date, 8, 2);
		$dateSortie .= "/";
		$dateSortie .= substr($date, 5, 2);
		$dateSortie .= "/";
		$dateSortie .= substr($date, 0, 4);
		
		if(strlen($date) > 10)
		{
			$dateSortie .= " ".substr($date, 11);
		}
		return $dateSortie;
	}
	
	public static function dateFrToEn($date)
	{
		$dateSortie = substr($date, 6, 4);
		$dateSortie .= "-";
		$dateSortie .= substr($date, 3, 2);
		$dateSortie .= "-";
		$dateSortie .= substr($date, 0, 2);
		
		if(strlen($date) > 10)
		{
			$dateSortie .= " ".substr($date, 11);
		}
		return $dateSortie;
	}
	
	public static function dateEnToTimestamp($date)
	{
		return mktime(substr($date, 11, 2), substr($date, 14, 2), substr($date, 17, 2), substr($date, 5, 2), substr($date, 8, 2), substr($date, 0, 4));
	}
	
	public static function dureeDifference($date_debut, $date_fin)
	{
		return date("H:i:s", (self::dateEnToTimestamp($date_fin) - self::dateEnToTimestamp($date_debut)) - 3600);
	}
}